@echo off

rem -----------------------------------------------
rem --- KapGen.cmd - By Marco Certelli - Beta06 ---
rem -----------------------------------------------

echo.

if "%1" EQU "-h" goto :PrintUsage
if "%1" EQU "-l" goto :ListMaps

goto :CheckTools
:ToolsOK

goto :CheckENV
:ENVOK

echo ************************************************
echo * KapGen: converting images maps to BSB format *
echo *   By Marco Certelli.  Use at your own risk   *
echo ************************************************
echo.

set MAP=%1
if "%MAP%" NEQ "" goto :NoInteractive

:Interactive
set MAP=
set /P MAP="Enter command [ -h | -l | -q | MAP ]: "

echo.

if "%MAP%" EQU "-h" goto :PrintUsage
if "%MAP%" EQU "-l" goto :ListMaps
if "%MAP%" EQU "-q" goto :EOF
if "%MAP%" EQU ""   goto :EOF

rem ************ Header File Retieval ***************
:NoInteractive

bin\mc2bsbh -s %MAP% Calibration\CHARTCAL.DIR

if not exist %MAP%.hdr copy Headers\%MAP%.hdr . 1>nul

if not exist %MAP%.hdr goto :HeaderMissing
echo.


rem ************* Map file Handling *****************

SET SRCMAP=
FOR /F "tokens=3 delims= " %%A IN ('findstr /C:"! IMGMOD " %MAP%.hdr') DO SET SRCMAP=%%A
if "%SRCMAP%" EQU "" SET SRCMAP=%MAP%.tif

set DOWNLOAD=0
if exist Img\%SRCMAP% goto :SourceAvailable

SET GETMAP=
FOR /F "tokens=3 delims= " %%A IN ('findstr /C:"! GETMAP " %MAP%.hdr') DO SET GETMAP=%%A
if "%GETMAP%" EQU "" goto :GETMAPMissing

bin\wget -O Img\%SRCMAP% %GETMAP%
if not exist Img\%SRCMAP% goto :DownloadMAPFailed
SET DOWNLOAD=1


rem *********** IMG Modification Handling ************
:SourceAvailable

SET IMGMOD=
FOR /F "tokens=4-8 delims= " %%A IN ('findstr /C:"! IMGMOD " %MAP%.hdr') DO SET IMGMOD=%%A %%B %%C %%D %%E
if "%IMGMOD%" EQU "" if "%SRCMAP%" EQU "%MAP%.tif" goto :NoImgModRequired

SET COLORS=0
SET IMGCUT=
FOR /F "tokens=1 delims= "   %%A IN ("%IMGMOD%") DO SET COLORS=%%A
FOR /F "tokens=2-5 delims= " %%A IN ("%IMGMOD%") DO SET IMGCUT=%%A %%B %%C %%D

SET MOD=
if "%SRCMAP%" EQU "%MAP%.tif" SET MOD=_mod

if  %COLORS%  GTR 64 SET COLORS=64

if "%IMGCUT%" NEQ "" if "%COLORS%" NEQ "0" bin\nconvert -colors %COLORS% -crop %IMGCUT% -out tiff -c 2 -o img\%MAP%%MOD%.tif img\%SRCMAP% 1>nul
if "%IMGCUT%" NEQ "" if "%COLORS%" EQU "0" bin\nconvert -crop %IMGCUT% -out tiff -c 2 -o img\%MAP%%MOD%.tif img\%SRCMAP% 1>nul
if "%IMGCUT%" EQU "" if "%COLORS%" NEQ "0" bin\nconvert -colors %COLORS% -out tiff -c 2 -o img\%MAP%%MOD%.tif img\%SRCMAP% 1>nul
if "%IMGCUT%" EQU "" if "%COLORS%" EQU "0" bin\nconvert -out tiff -c 2 -o img\%MAP%%MOD%.tif img\%SRCMAP% 1>nul
echo.


rem ************** MD5 Check ********************
:NoImgModRequired

SET MD5SUM=
FOR /F "tokens=3 delims= " %%A IN ('findstr /C:"! MD5SUM " %MAP%.hdr') DO SET MD5HDR=%%A
if "%MD5HDR%" EQU "" goto :MD5Missing

SET MD5IMG=
FOR /F "tokens=1 delims= " %%A IN ('bin\md5sums -u img\%SRCMAP%') DO set MD5IMG=%%A
if "%MD5IMG%" NEQ "%MD5HDR%" goto :MD5Error

echo MD5 Verified! Header calibration data apply to %SRCMAP%
echo.


rem ************** TIF 2 BSB ********************

bin\tif2bsb %MAP%.hdr img\%MAP%%MOD%.tif BSB\%MAP%.kap

if not exist BSB\%MAP%.kap goto :BSBConvFailed

echo.
echo SUCCESS! New map %MAP%.kap is stored in BSB\ folder.
echo.
echo MAP copyright (source image ^& calibration data):

FOR /F "tokens=2,*" %%A IN ('findstr /C:"! MAPCPR " %MAP%.hdr') DO echo %%B
FOR /F "tokens=2,*" %%A IN ('findstr /C:"! CALCPR " %MAP%.hdr') DO echo %%B
echo.
FOR /F "tokens=2,*" %%A IN ('findstr /C:"! WARNING" %MAP%.hdr') DO echo %%A %%B
goto Pause


rem ************* Error Handling *****************

:HeaderMissing
echo ERROR: %MAP% header not found neither in CHARTCAL.DIR nor in Headers\
echo        Please note that search in CHARTCAL.DIR is case sensitive
goto Pause


:GETMAPMissing
echo ERROR: The %MAP% header does not say how to automatically get %SRCMAP%
:ManualDownload
echo        Obtain the file map manually, put in img\ and restart KapGen
echo.
echo %MAP% header comments may help...:
findstr /B "!" %MAP%.hdr
goto Pause


:DownloadMAPFailed
echo ERROR: Automatic Download of %SRCMAP% didn't work
goto :ManualDownload


:MD5Missing
echo ERROR: %MAP% header does't contain a valid MD5. Consistency of calibration
echo        data vs. map file cannot be assured. KapGen cannot continue.
goto Pause


:MD5Error
echo ERROR: This %MAP%.hdr cannot be used with %SRCMAP% for a MD5 mismatch!
if %DOWNLOAD% EQU 0 echo        Maybe Img/%SRCMAP% is older or newer than calibration data
if %DOWNLOAD% EQU 0 echo        Obtain a new file (if older) or calibrate this one (if newer)
if %DOWNLOAD% EQU 1 echo        Dowloaded %SRCMAP% is maybe new: calibrate this or get a new header

if exist Calibration\%MAP%.png goto :PNGReady
echo.
echo Generating %MAP%.png image (in case a re-calibration is required)
echo.
bin\nconvert -out png -o Calibration\%MAP%.png Img\%MAP%.tif 1>nul

:PNGReady
echo.
echo Calibration\%MAP%.png is Ready. Use MapCal-II for update CHARTCAL.DIR
echo After calibration, put following MD5 in CHARTCAL.DIR, section [%MAP%.png]
echo.
bin\md5sums -u Img\%SRCMAP%
goto Pause


:BSBConvFailed
echo ERROR: The TIF to BSB conversion has failed (unknown reason)...
goto Pause


rem ************* Switch Handling *****************

:PrintUsage
echo Usage: KapGen [-h ^| -l ^| -d ^| MAP]
echo.
echo        -h     : print this help...
echo        -l     : list for available MAPs ^(headers^)
echo        -q     : Exit the interactive mode (or just press Return)
echo        MAP    : name of the map to convert ^(header without .hdr^)
echo.
echo Example: KapGen NZ21  : will generate NZ21.kap using NZ21.hdr
echo          KapGen       : will enter the interactive mode
goto Pause


:ListMaps
echo :: List of calibrations in CHARTCAL.DIR ::
bin\mc2bsbh -l Calibration\CHARTCAL.DIR | sort
if not exist Headers\*.hdr goto :Pause
echo.
echo :: List of headers in Headers/ folder ::
for %%I in (Headers\*.hdr) DO echo %%~nI
goto :Pause


rem ************* Tools Presence check *****************

:CheckTools
SET error=0

if not exist bin\nconvert.exe set error=1

if not exist bin\md5sums.exe  set error=1

if not exist bin\mc2bsbh.exe  set error=1

if not exist bin\tif2bsb.exe  set error=1

if not exist bin\unzip.exe    set error=1      

if %error% EQU 0 Goto :ToolsOK

echo.
echo Some tools needed to KapGen are missing.
echo Download the full KapGen package at http://mce66.altervista.org/software.html
echo.
echo Type any key then Return to open the browser at the D/L page,

SET YN=
SET /P YN="press only Return to exit: "

if "%YN%" EQU "" goto :EOF

echo.
echo Please wait a few seconds...

start http://mce66.altervista.org/software.html#Free_Nautical_Raster_Maps

goto :EOF

rem ************* Environment check *****************

:CheckENV
if not exist BSB/ mkdir BSB
if not exist Headers/ mkdir Headers
if not exist Calibration/ mkdir Calibration
if not exist Img/ mkdir Img

if exist Calibration/CHARTCAL.DIR goto :ENVOK
echo.
echo ERROR: Calibration/CHARTCAL.DIR file is missing. Get one and restart KapGen
echo        One can be download at http://mce66.altervista.org/software.html
goto Pause

rem ****************** Exiting *******************

:Pause
del %MAP%.hdr 2>nul
if "%1" NEQ "" goto :EOF
echo.
goto :Interactive
